// Datum.java                                  MuP1 2014/15

/**
 * Klasse Datum.
 */
public class Datum
{
/* ----------------------------------------------------- */
                                               // Attribute
/**
 * Tag (1..28/29/30/31)
 */
  protected int tag;

/**
 * Monat (1..12)
 */
  protected int monat;

/**
 * Jahr (1600..3000)
 */
  protected int jahr;
  
/**
 * Wochentag (Sonntag..Samstag)
 */
  private String wochenTag = "";

/**
 * Schaltjahr.
 */
  private boolean schaltJahr = false;

/* ----------------------------------------------------- */
                                             // set-Methode
/**
 * Setzt Datum.
 * @param tt Tag des Datums
 * @param mm Monat des Datums
 * @param ccjj Jahr des Datums
 */
  public void setDatum( int tt, int mm, int ccjj)
    throws DatumException
  {
// berprfen der Datumseingabe
                                   
    jahr = 0;                             // Datum loeschen
    monat = 0;
    tag = 0;

    if( ccjj < 1600 || ccjj > 3000)                 // Jahr
      throw new DatumException();

    if( mm < 1 || mm > 12)                         // Monat
      throw new DatumException();
 
    boolean schaltJahr;                       // Schaltjahr
    int cc = ccjj / 100;
    int jj = ccjj % 100;
    if( jj == 0) schaltJahr = cc % 4 == 0;
    else  schaltJahr = jj % 4 == 0;

    int anzahl = 0;             // Anzahl der Tage im Monat
    switch( mm)
    {
      case 1: case 3: case 5: case 7: case 8: case 10:
      case 12: anzahl = 31; break;

      case 4: case 6: case 9:
      case 11: anzahl = 30; break;

      case 2: if( schaltJahr) anzahl = 29;
              else anzahl = 28;
    }

    if( tt < 1 || tt > anzahl)                       // Tag
      throw new DatumException();

    jahr = ccjj;                           // Datum korrekt
    monat = mm;
    tag = tt;
    
    berechneWochenTag();
  }

/* ----------------------------------------------------- */
                                     // Wochentag berechnen
/**
 * Berechne Wochentag nach W. Jacobsthal (1917).
 * Ausgangsdatum der Berechnung: Sonntag, 1.5.1600
 */
  private void berechneWochenTag()
  {
    int cc = jahr / 100;                      // Schaltjahr
    int jj = jahr % 100;
    if( jj == 0) schaltJahr = cc % 4 == 0;
    else  schaltJahr = jj % 4 == 0;

    int korrektur = 0;                     // Korrekturtage
    switch( monat)
    {
      case  1: korrektur = schaltJahr? 5 : 6; break;
      case  2: korrektur = schaltJahr? 1 : 2 ; break;
      case  3: korrektur = 2; break;
      case  4: korrektur = 5; break;
      case  5: korrektur = 0; break;
      case  6: korrektur = 3; break;
      case  7: korrektur = 5; break;
      case  8: korrektur = 1; break;
      case  9: korrektur = 4; break;
      case 10: korrektur = 6 ; break;
      case 11: korrektur = 2 ; break;
      case 12: korrektur = 4 ; break;
    }

    int korrekturTage
    = tag + korrektur + jj + jj / 4 - 2 * ( cc % 4);

    switch( korrekturTage % 7)                 // Wochentag
    {
      case 0: wochenTag = "Sonntag"; break;
      case 1: wochenTag = "Montag"; break;
      case 2: wochenTag = "Dienstag"; break;
      case 3: wochenTag = "Mittwoch"; break;
      case 4: wochenTag = "Donnerstag"; break;
      case 5: wochenTag = "Freitag"; break;
      case 6: wochenTag = "Samstag"; break;
    }
  }
/* ----------------------------------------------------- */
                                                // toString
/**
 * Datum als String "Wochentag, den tt.mm.ccjj".
 * @return Datum
 */
  public String toString()
  {
    return wochenTag + ", den "
           + tag + ". " + monat + ". " + jahr;
  }
}
